from ..utils.microservices import Microservice
from ..utils.exceptions import MsAuthException
from ..utils.request import Request, GET
from ..utils.error import errorBreakApi
from .kms import KmsGateway
from json import loads


class AuthGateway:

    @classmethod
    def getPrivateKey(cls, language):
        response = Request.fetch(
            url="{baseUrl}/private-key".format(baseUrl=Microservice.getBaseUrl(Microservice.auth)),
            method=GET,
            deadline=50
        )

        if response.status == 400:
            return None, response.contentJson()
        elif response.status > 200:
            return None, errorBreakApi(language=language)

        return response.contentJson().get("privateKey"), None

    @classmethod
    def getExternalCredentials(cls, credentialsId):
        response = Request.fetch(
            url="{baseUrl}/external-credentials/{externalCredentialsId}".format(
                baseUrl=Microservice.getBaseUrl(Microservice.auth),
                externalCredentialsId=credentialsId,
            )
        )

        if response.status == 400:
            raise MsAuthException(response.contentJson())
        if response.status != 200:
            raise MsAuthException(response.content)

        body = response.contentJson()
        encryptedCredentials = body["encryptedData"]
        keyId = body["keyId"]

        rawCredentials = KmsGateway.decrypt(encryptedCredentials, keyId)
        return rawCredentials

    @classmethod
    def getJsonCredentials(cls, credentialsId):
        return loads(cls.getExternalCredentials(credentialsId))
