from ..utils.microservices import Microservice
from ..utils.request import Request


class WorkspaceGateway:

    @classmethod
    def getOwner(cls, workspaceId, headers):
        response = Request.fetch(
            url="{baseUrl}/workspace/{workspaceId}/owner".format(baseUrl=Microservice.getBaseUrl(Microservice.workspace), workspaceId=workspaceId),
            headers=headers
        )

        if response.status > 400:
            raise Exception("Unable to contact with ms-workspace http status {status} to get workspace owner".format(status=response.status))

        if response.status == 400:
            return None, response.contentJson()

        return response.contentJson()["owner"], None

    @classmethod
    def getOwners(cls, workspaceIds, headers):
        response = Request.fetch(
            url="{baseUrl}/owner?workspaceIds={workspaceIds}".format(
                baseUrl=Microservice.getBaseUrl(Microservice.workspace),
                workspaceIds=",".join(workspaceIds)
            ),
            headers=headers
        )

        if response.status > 400:
            raise Exception("Unable to contact with ms-workspace http status {status} to get workspace owner".format(status=response.status))

        if response.status == 400:
            return None, response.contentJson()

        return response.contentJson()["owners"], None

    @classmethod
    def getById(cls, workspaceId, headers):
        response = Request.fetch(
            url="{baseUrl}/workspace/{workspaceId}".format(baseUrl=Microservice.getBaseUrl(Microservice.workspace), workspaceId=workspaceId),
            headers=headers
        )

        if response.status > 400:
            raise Exception("Unable to contact with ms-workspace http status {status} to get workspace by id from workspaceId {workspaceId}".format(status=response.status, workspaceId=workspaceId))

        if response.status == 400:
            return None, response.contentJson()

        return response.contentJson()["workspace"], None

    @classmethod
    def getMemberInfo(cls, memberId, headers):
        response = Request.fetch(
            url="{baseUrl}/member/{memberId}".format(baseUrl=Microservice.getBaseUrl(Microservice.workspace), memberId=memberId),
            headers=headers
        )

        if response.status > 400:
            raise Exception(
                "Unable to contact with ms-workspace http status {status} to get member by id for memberId {memberId}".format(
                    status=response.status, memberId=memberId))

        if response.status == 400:
            return None, response.contentJson()

        return response.contentJson()["member"], None
