from datetime import datetime, timedelta
from ..utils.apiMessage import SharedApiCode, SharedApiMessageKey
from ..utils.date import Date


def validateDateString(date, format="%Y-%m-%d"):
    if not date:
        return None
    try:
        return datetime.strptime(date, format)
    except:
        return None


def validateOptionalAfterBefore(func):
    def wrapper(self, **data):
        afterString = self.getParameter("after")
        beforeString = self.getParameter("before")

        after = validateDateString(afterString)
        before = validateDateString(beforeString)

        if afterString and not after:
            return self.sendJson(
                self.responseJson().error(
                    code=SharedApiCode.invalidDate,
                    messageKey=SharedApiMessageKey.invalidParameter,
                    parameter="after"
                )
            )
        
        if beforeString and not before:
            return self.sendJson(
                self.responseJson().error(
                    code=SharedApiCode.invalidDate,
                    messageKey=SharedApiMessageKey.invalidParameter,
                    parameter="before"
                )
            )

        if after and before and after > before:
            return self.sendJson(
                self.responseJson().error(
                    code=SharedApiCode.invalidDate,
                    messageKey=SharedApiMessageKey.invalidDateOrder
                )
            )

        data.update({
            "after": Date.utcFromTimezone(after) if after else None,
            "before": Date.utcFromTimezone(before) + timedelta(days=1) - timedelta(seconds=1) if before else None
        })

        return func(self, **data)
    return wrapper
