from datetime import datetime, timedelta
from ..utils.apiMessage import SharedApiCode, SharedApiMessageKey
from ..utils.enum import Enum


class TimeFormats(Enum):
    dateFormat = "%Y-%m-%d"
    fullSecondDatetimeFormat = "%Y-%m-%d %H:%M:%S"
    subSecondDatetimeFormat = "%Y-%m-%d %H:%M:%S.%f"



def _validateDateString(date):
    if not date:
        return None, False

    try:
        return datetime.strptime(date, TimeFormats.dateFormat), False
    except:
        pass

    try:
        return datetime.strptime(date, TimeFormats.fullSecondDatetimeFormat), False
    except:
        pass

    try:
        return datetime.strptime(date, TimeFormats.subSecondDatetimeFormat), False
    except:
        pass

    return None, True


def validateOptionalAfterBeforeDatetime(after, before):
    def decorator(func):
        def wrapper(self, **data):
            afterValue, error = _validateDateString(self.getParameter(after))
            if error:
                return self.sendJson(
                    self.responseJson().error(
                        code=SharedApiCode.invalidDateTime,
                        messageKey=SharedApiMessageKey.invalidDateTime,
                        parameter=after,
                        formats=TimeFormats.values(),
                    )
                )

            beforeValue, error = _validateDateString(self.getParameter(before))
            if error:
                return self.sendJson(
                    self.responseJson().error(
                        code=SharedApiCode.invalidDateTime,
                        messageKey=SharedApiMessageKey.invalidDateTime,
                        parameter=before,
                        formats=TimeFormats.values(),
                    )
                )

            if afterValue and beforeValue and afterValue > beforeValue:
                return self.sendJson(
                    self.responseJson().error(
                        code=SharedApiCode.invalidDate,
                        messageKey=SharedApiMessageKey.invalidDateOrder
                    )
                )

            data.update({
                after: afterValue if afterValue else None,
                before: beforeValue if beforeValue else None,
            })

            return func(self, **data)

        return wrapper

    return decorator
