from ..utils.apiMessage import SharedApiCode, SharedApiMessageKey
from functools import wraps


def validateId(parameterName):
    def decorator(func):
        def wrapper(self, **data):
            idParameter = self.getParameters(parameterName)

            try:
                idParameter = str(int(idParameter))
            except:
                return self.sendJson(
                    self.responseJson().error(
                        code=SharedApiCode.invalidIds,
                        messageKey=SharedApiMessageKey.invalidIdType,
                        parameterName=parameterName,
                    )
                )

            data[parameterName] = idParameter

            return func(self, **data)

        return wrapper

    return decorator


def validateEntityIds(parameterName):
    def decorator(func):
        def wrapper(self, **data):
            entityIds = self.getParameter(parameterName)

            if entityIds:
                try:
                    entityIds = [str(int(entityId)) for entityId in entityIds]
                except:
                    return self.sendJson(
                        self.responseJson().error(
                            code=SharedApiCode.invalidIds,
                            messageKey=SharedApiMessageKey.badIdsFormatting,
                            idsParameterName=parameterName,
                        )
                    )

            data[parameterName] = entityIds

            return func(self, **data)

        return wrapper

    return decorator


def validateIdsParameter(gateway, required, field="ids"):
    """Parse `ids` query parameter as a comma-separated list of ChargePayment ids."""

    def decorator(func):
        @wraps(func)
        def wrapper(self, **data):
            paymentIds = set([
                paymentId.strip()
                for paymentId in self.getParameter("ids").split(",")
                if paymentId
            ])

            if required and not paymentIds:
                return self.sendJson(
                    self.responseJson().error(
                        code=SharedApiCode.invalidAction,
                        messageKey=SharedApiMessageKey.missingParameter,
                        parameter="ids",
                    )
                )

            payments, error = _handleMultipleIds(self, paymentIds, gateway)
            if error:
                return self.sendJson(error)

            data.update({
                field: payments
            })

            return func(self, **data)

        return wrapper

    return decorator


def _handleMultipleIds(self, paymentIds, gateway):
    payments = {
        payment.id(): payment
        for payment in gateway.getByIds(paymentIds)
        if payment
    }

    errors = []
    for paymentId in paymentIds:
        payment = payments.get(paymentId)
        if not payment or payment.workspaceId != self.workspaceId():
            errors.append(
                self.responseJson().errorElement(
                    code=SharedApiCode.invalidIds,
                    messageKey=SharedApiMessageKey.invalidParameter,
                    index=paymentId,
                    parameter="ids",
                )
            )

        if payment and payment.deletedBy:
            errors.append(
                self.responseJson().errorElement(
                    code=SharedApiCode.invalidIds,
                    messageKey=SharedApiMessageKey.deletedEntity
                )
            )

    if len(errors) > 0:
        return None, self.responseJson().errors(errors=errors)

    return list(payments.values()), None
