from re import sub
from ..utils.identifier import Identifier
from ..utils.apiMessage import SharedApiMessageKey, SharedApiCode
from ..utils.string import String
from ..utils.cnpj import Cnpj
from ..utils.cpf import Cpf


def validateOptionalTaxId(func):
    def wrapper(self, **data):
        taxId = self.getParameter("taxId")
        if not taxId:
            data.update({"taxId": None})
            return func(self, **data)

        errors = isValidTaxId(self, taxId)
        if errors:
            return self.sendJson(
                self.responseJson().errors(errors=errors)
            )

        data.update({
            "taxId": Identifier.format(taxId)
        })

        return func(self, **data)
    return wrapper


def isValidTaxId(self, taxId):
    if not String.isString(taxId):
        return [self.responseJson().errorElement(
            code=SharedApiCode.invalidTaxId,
            messageKey=SharedApiMessageKey.notString
        )]

    taxIdOnlyNumbers = sub(r"\D", "", taxId)

    if len(taxIdOnlyNumbers) not in (11, 14):
        return [self.responseJson().errorElement(
                code=SharedApiCode.invalidTaxId,
                messageKey=SharedApiMessageKey.invalidTaxId
            )]

    errors = []

    if len(taxIdOnlyNumbers) == 11:
        if not Cpf(taxId).matchPattern():
            errors.append(
                self.responseJson().errorElement(
                    code=SharedApiCode.invalidTaxId,
                    messageKey=SharedApiMessageKey.notMatchRegex,
                    parameter="taxId",
                    regex=Cpf.regex
                )
            )

        if not Cpf(taxIdOnlyNumbers).isValid():
            errors.append(
                self.responseJson().errorElement(
                    code=SharedApiCode.invalidTaxId,
                    messageKey=SharedApiMessageKey.invalidCpf
                )
            )

    if len(taxIdOnlyNumbers) == 14:
        if not Cnpj(taxId).matchPattern():
            errors.append(
                self.responseJson().errorElement(
                    code=SharedApiCode.invalidTaxId,
                    messageKey=SharedApiMessageKey.notMatchRegex,
                    parameter="taxId",
                    regex=Cnpj.regex
                )
            )

        if not Cnpj(taxIdOnlyNumbers).isValid():
            errors.append(
                self.responseJson().errorElement(
                    code=SharedApiCode.invalidTaxId,
                    messageKey=SharedApiMessageKey.invalidCnpj
                )
            )

    return errors
