from datetime import datetime
from .date import Date
from .email import Email
from .enum import Enum
from .metadata import Metadata
from .microservices import Environment
from .slack import Slack
from ..gateways.alert import AlertGateway


class AlertLevel(Enum):

    notification = "notification"
    warning = "warning"
    critical = "critical"


class Alert:

    @classmethod
    def send(cls, text, level=AlertLevel.warning, cooldown=3*3600):
        service = Metadata.getProjectId()

        if AlertGateway.getAlertInMemcache(service=service, level=level, content=text):
            return

        Slack.send(
            sender=service,
            text=_slackMessageTemplates[level].format(message=text),
            channel=_slackChannels[Environment.current()]
        )

        AlertGateway.saveAlertInMemcache(service=service, level=level, content=text, cooldown=cooldown)

        Email.send(
            to="developers@starkbank.com",
            text=text,
            subject="[{environment}] [{service}] {date}".format(
                environment=Environment.current(),
                service=service,
                date=Date.utcToTimezone(datetime.utcnow()).strftime("%Y-%m-%d %H:%M"),
            )
        )

        return


_slackMessageTemplates = {
    AlertLevel.notification: ":mailbox_with_mail: *Notification* :mailbox_with_mail:\t{message}",
    AlertLevel.warning:      ":warning:    *Warning*    :warning:\t{message}",
    AlertLevel.critical:     ":fire:     *Critical*     :fire:\t{message}",
}

_slackChannels = {
    Environment.development: "#alerts-dev",
    Environment.sandbox:     "#alerts-sbx",
    Environment.production:  "#alerts-prd"
}
