

class CsvParser:

    def __init__(self, text):
        self.text = text.strip()
        self.rows = self.text.splitlines()
        commaCount = text.count(",")
        semicolonCount = text.count(";")
        if "sep=" in self.rows[0]:
            self.delimiter = self.rows[0][4]
            self.rows.pop(0)
        else:
            self.delimiter = "," if commaCount >= semicolonCount else ";"

    def headers(self):
        return self.rows[0].split(self.delimiter)

    def dict(self):
        result = []
        for row in self.rows[1:]:
            arrayOfValues = CsvParser.mergeQuotes(array=row.split(self.delimiter))
            result.append({header.strip(): value.strip() for (header, value) in zip(self.headers(), arrayOfValues)})

        return result

    @classmethod
    def mergeQuotes(cls, array, delimiter=","):
        start = -1
        content = []
        for index, string in enumerate(array):
            string = string.strip()
            if string and string[0] == "\"":
                start = index
                content.append(string[1:].strip())
            if content and "\"" not in string:
                array[index] = None
                content.append(string.strip())
            if string and string[-1] == "\"":
                array[index] = None
                content.append(string[:len(string) - 1].strip())
                array[start] = delimiter.join(content)
                content = []
        return [cls.forceEncode(string) for string in array if string is not None]

    @classmethod
    def forceEncode(cls, string, codecs=None, encode="utf8"):
        encodingArray = codecs or ["utf8", "latin-1"]
        for encodingType in encodingArray:
            try:
                return string.decode(encodingType).encode(encode)
            except UnicodeDecodeError:
                continue

        raise Exception("Error when decoding csv!")
