from enum import Enum
from nationalHolidays import holidays
from pytz import utc, timezone


class Date:

    @classmethod
    def utcToIsoformat(cls, datetime):
        """
        This method convert a UTC date to string in isoformat

        :param datetime: UTC date

        :return: string in isoformat
        """
        return utc.localize(datetime).isoformat()

    @classmethod
    def utcToTimezone(cls, datetime, zone="America/Sao_Paulo"):
        """
        This method converts a date in UTC to a new timezone

        :param datetime: date in UTC
        :param zone: desired timezone from conversion

        :return: date in the desired timezone
        """
        tz = timezone(zone)
        return utc.localize(datetime).astimezone(tz).replace(tzinfo=None)

    @classmethod
    def utcFromTimezone(cls, datetime, zone="America/Sao_Paulo"):
        """
        This method converts a date in timezone to UTC

        :param datetime: date in zone
        :param zone: current timezone

        :return: date in UTC
        """
        tz = timezone(zone)
        return tz.normalize(tz.localize(datetime)).astimezone(utc).replace(tzinfo=None)


class Week(Enum):

    monday = 0
    tuesday = 1
    wednesday = 2
    thursday = 3
    friday = 4
    saturday = 5
    sunday = 6


def isWeekend(date):
    return date.weekday() in (Week.saturday, Week.sunday)


def isHoliday(date):
    return date in holidays
