import re, urllib, logging
from base64 import encodestring
from ..gateways.auth import AuthGateway
from ..utils.request import Request, POST
from google.appengine.api import mail


class Email:

    @classmethod
    def isValid(cls, email):
        regex = re.compile("^[A-Z0-9a-z._%+-]+@[A-Za-z0-9.-]+\\.[A-Za-z]{2,64}")
        return True if regex.match(email) else False

    @classmethod
    def send(cls, to, subject, sender="Stark Bank<help@starkbank.com>", text=None, html=None, attachments=None, replyTo=None):
        try:
            cls.sendEmailByAppEngine(
                text=text,
                html=html,
                to=to,
                subject=subject,
                sender=sender,
                attachments=attachments
            )
        except:
            cls.sendEmailByMailGun(
                text=text,
                html=html,
                to=to,
                subject=subject,
                sender=sender,
                attachments=attachments,
                replyTo=replyTo
            )

    @classmethod
    def sendEmailByAppEngine(cls, to, subject, sender, text=None, html=None, attachments=None):
        message = mail.EmailMessage()
        message.sender = sender
        message.to = to
        message.subject = subject
        if text:
            message.body = text
        if html:
            message.html = html
        if attachments:
            message.attachments = attachments
        return message.send()

    @classmethod
    def sendEmailByMailGun(cls, to, subject, sender, text=None, html=None, attachments=None, replyTo=None):
        data = {
            "to": to,
            "from": sender,
            "subject": subject
        }
        if text:
            data["text"] = text
        if html:
            data["html"] = html.encode("utf-8")
        if attachments:
            data["attachment"] = attachments
        if replyTo:
            data["h:Reply-To"] = replyTo

        credentials = AuthGateway.getJsonCredentials("mailgun")

        response = Request.fetch(
            url="https://api.mailgun.net/v3/{domain}/messages".format(domain=credentials["domain"]),
            method=POST,
            body=urllib.urlencode(data),
            headers={"Authorization": "Basic %s" % encodestring(credentials["apiKey"])},
        )

        if response.status != 200:
            logging.error(response.content)

        return response.status == 200
