import sys
from os import getenv
try:
    import requests
except:
    pass


class Metadata:

    @classmethod
    def getProjectId(cls):
        try:
            from google.appengine.api import app_identity
            return app_identity.get_application_id()
        except ImportError:
            pass
        try:
            url = "http://metadata.google.internal/computeMetadata/v1/project/project-id"
            headers = {"Metadata-Flavor": "Google"}
            return requests.get(url, headers=headers).text
        except requests.exceptions.ConnectionError:
            return [s for s in sys.path[0].split("/") if "ms-" in s][0]

    @classmethod
    def getRunningVersion(cls):
        version = getenv("CURRENT_VERSION_ID", ".")
        return version.split(".")[0]

    @classmethod
    def getServiceAccessToken(cls, scope="https://www.googleapis.com/auth/cloud-platform"):
        try:
            from google.appengine.api import app_identity
            return app_identity.get_access_token(scope)[0]
        except ImportError:
            pass

        try:
            url = "http://metadata.google.internal/computeMetadata/v1/instance/service-accounts/default/token"
            headers = {"Metadata-Flavor": "Google"}
            return requests.get(url, headers=headers).json()["access_token"]
        except requests.exceptions.ConnectionError:
            return [s for s in sys.path[0].split("/") if "ms-" in s][0]
