from os import environ
from cloudstorage import open, delete, stat, listbucket, copy2, NotFoundError
from google.appengine.api import app_identity


class Storage:

    @classmethod
    def defaulfBucket(cls):
        return environ.get("", app_identity.get_default_gcs_bucket_name())

    @classmethod
    def bucketPath(cls, fileName="", filePath="", bucketName=None):
        bucketName = bucketName if bucketName is not None else cls.defaulfBucket()
        return "/" + bucketName + filePath + "/" + fileName

    @classmethod
    def baseURL(cls):
        return "https://storage.googleapis.com"

    @classmethod
    def baseURI(cls):
        return "gs:/"

    @classmethod
    def publicUrl(cls, bucketPath):
        return str(cls.baseURL() + bucketPath)

    @classmethod
    def publicUri(cls, bucketPath):
        return str(cls.baseURI() + bucketPath)

    @classmethod
    def save(cls, file, bucketPath, contentType=None, isPublic=False, metadata=None, returnUri=False):
        options = {"x-goog-acl": "public-read"} if isPublic else {}
        if metadata is not None:
            options.update({
                str("x-goog-meta-" + key): str(value)
                for key, value in metadata.items()
            })
        gcsFile = open(bucketPath, "w", content_type=contentType, options=options)
        gcsFile.write(file)
        gcsFile.close()
        if returnUri:
            return cls.publicUri(bucketPath)
        return cls.publicUrl(bucketPath)

    @classmethod
    def openFile(cls, bucketPath):
        return open(bucketPath)

    @classmethod
    def closeFile(cls, gcsFile):
        gcsFile.close()
        return

    @classmethod
    def read(cls, bucketPath):
        gcsFile = open(bucketPath)
        content = gcsFile.read()
        gcsFile.close()
        return content

    @classmethod
    def readPartial(cls, bucketPath, offset, size):
        gcsFile = open(bucketPath)
        gcsFile.seek(offset, 0)
        content = gcsFile.read(size)
        gcsFile.close()
        return content

    @classmethod
    def delete(cls, bucketPath):
        try:
            delete(bucketPath)
        except NotFoundError:
            pass

    @classmethod
    def exists(cls, bucketPath):
        try:
            stat(bucketPath)
            return True
        except:
            return False

    @classmethod
    def list(cls, path):
        for bucket in listbucket(path):
            yield bucket.filename

    @classmethod
    def move(cls, currentFilePath, newFolderPath):
        fileName = currentFilePath.split('/')[-1]
        newFilePath = newFolderPath + fileName
        copy2(src=currentFilePath, dst=newFilePath)
        delete(currentFilePath)

    @classmethod
    def getSize(cls, bucketPath):
        return stat(bucketPath).st_size

    @classmethod
    def getMetadata(cls, bucketPath):
        try:
            allMetadata = stat(bucketPath).metadata
        except NotFoundError:
            return None
        return {
            key[12:]: value
            for key, value in allMetadata.iteritems()
            if key.startswith("x-goog-meta-")
        }
